
#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "runcmd.h"
#include "options.h"
#include "version.h"

extern char **environ;

/* Runs a command and redirects stdin and stdout to already created (we hope ;) files. */
int re_system_internal (const char *command,const char *stdin_file,const char *stdout_file) {
  int pid;
  char *argv[4];
  char *tmpstr,*tmpstr2;

  if (!daemon_mode)
    printf("Running command %s\n",command);

  if (command == 0)
    return 1;
  pid = fork();
  if (pid == -1)
    return -1;
  if (pid == 0) {

    /* Open the stdin and stdout pipes. */
    freopen(stdin_file,"w+",stdin);
    freopen(stdout_file,"w+",stdout);
    freopen(stdout_file,"w+",stderr);

    chmod(stdout_file,S_IREAD | S_IWRITE);

    tmpstr = strdup(getenv("PATH"));

    tmpstr2 = malloc(strlen("PATH=" PLUGIN_PATH ":") + strlen(tmpstr) + 1);
    tmpstr2 = strcpy(tmpstr2,"PATH=" PLUGIN_PATH ":");
    tmpstr2 = strcat(tmpstr2,tmpstr);

    putenv(tmpstr2);

    argv[0] = "sh";
    argv[1] = "-c";
    argv[2] = (char *)command;
    argv[3] = 0;
    execve("/bin/sh", argv, environ);

    /* Looks like an error happened. Better write a report. */
    printf("BEGIN %s\n",monitorversion);
    printf("URGENCY 5\n");
    printf("REPORT\n");
    #if HAVE_STRERROR
    printf("Error while running %s - %s\n",command,strerror(errno));
    #else
    printf("Error while running %s\n",command);
    #endif
    printf("DONE\n");
    exit(127);
  }

  return pid;
}

/* Runs a command and redirects stdin to a FIFO and stdout to a file. */
int re_system(const char *command,const char *stdin_file,const char *stdout_file){
  /* Make a FIFO for stdin. */
  mknod(stdin_file,S_IFIFO,0);

  /* Set the permissions on the new pipes. */
  chmod(stdin_file,S_IREAD | S_IWRITE);

  /* Call re_system_internal to actually run the command. */
  return re_system_internal(command,stdin_file,stdout_file);
}

/* Runs a command and redirects stdin and stdout to a FIFO. */
int re_system2(const char *command,const char *stdin_file,const char *stdout_file){
  /* Make a FIFO for stdin and stdout. */
  mknod(stdin_file,S_IFIFO,0);
  mknod(stdout_file,S_IFIFO,0);

  /* Set the permissions on the new pipes. */
  chmod(stdin_file,S_IREAD | S_IWRITE);
  chmod(stdout_file,S_IREAD | S_IWRITE);

  /* Call re_system_internal to actually run the command. */
  return re_system_internal(command,stdin_file,stdout_file);
}

static char *id="@(#) $Id: runcmd.c,v 4.0 1999/10/07 22:56:15 pete Exp $";

/* 
 * $Log: runcmd.c,v $
 * Revision 4.0  1999/10/07 22:56:15  pete
 * Started work on version 1.0
 *
 * Revision 3.3  1999/09/20 19:46:48  pete
 * Now directly changes the path in the enviornment with putenv
 *
 * Revision 3.2  1999/09/19 18:25:57  pete
 * Added support for automake
 *
 * Revision 3.1  1999/09/19 16:11:29  pete
 * Now supports the HAVE_STRERROR #define
 *
 * Revision 3.0  1999/09/19 01:15:25  pete
 * Started work on 0.03
 *
 * Revision 2.4  1999/09/17 23:42:31  pete
 * Fixed a string handling bug.
 *
 * Revision 2.3  1999/09/16 21:55:14  pete
 * Added re_system2 function. Now re_system and re_system2 create the stdin
 * and stdout files/FIFOs and then call re_system_internal to actually run
 * the proccess.
 *
 * Revision 2.2  1999/09/16 18:56:17  pete
 * Now spawns childs with the same environment as parent.
 *
 * Revision 2.1  1999/09/15 23:20:44  pete
 * *** empty log message ***
 *
 * Revision 2.0  1999/09/15 16:05:52  pete
 * Started work on version 0.02
 *
 * Revision 1.13  1999/09/13 00:05:20  pete
 * Now sets the path so the plugin directory is checked first.
 *
 * Revision 1.12  1999/09/12 13:26:16  pete
 * Bug fix. :(
 *
 * Revision 1.11  1999/09/12 13:25:38  pete
 * Fixed the fork fail report to include the version number on BEGIN
 *
 * Revision 1.10  1999/09/11 19:46:20  pete
 * Now redirects stderr to stdout_file
 *
 * Revision 1.9  1999/09/11 19:44:56  pete
 * Added debugging for when a execve fails.
 *
 * Revision 1.8  1999/09/11 02:07:34  pete
 * Added ID and LOG tags after much work. :)
 *
 */
